/**
* @file i2c_software_master.c
*
* @brief I2C software master source code
*
* @author  Artur Funk  a_funk80@yahoo.de
*
* @date  16.11.2007
*/


#ifndef 	F_CPU				
#define 	F_CPU 28000000UL
#endif

#include <avr/io.h>
#include <util/delay.h>
#include "kernel/uzebox.h"

/** 
 @brief register definition for I2C communication
*/
#define I2C_DDR			DDRD					// Register fr die I2C Kommunikation

/** 
 @brief SDA port definiton to set port direction
*/
#define I2C_DDR_REG_BIT	DDD5	

/** 
 @brief I2C output port register
*/
#define I2C_PORT		PORTD					

/** 
 @brief I2C clock port (SCL) definition
*/
#define SCL				PD3						// Port D Pin 3

/** 
 @brief I2C data port (SDA) definition
*/
#define SDA				PD5						// Port D Pin 5

/** 
 @brief I2C input port register
*/
#define SDA_LESE_PIN 	PIND

/** 
 @brief I2C input port signal
*/
#define SDA_LESE_BIT 	SDA

/**
set data signal low (output)
@param void
@return  none
*/
void sda_low()
{
	I2C_PORT 		&=  ~(1<<SDA);    			// internal Pull-Up off
	I2C_DDR 		|=   (1<<I2C_DDR_REG_BIT);  // set SDA pin as output

}

/**
set data signal high (input)
@param void
@return  none
*/
void sda_high()
{
	I2C_DDR 		&= ~(1<<I2C_DDR_REG_BIT);  	// set SDA pin as input
	I2C_PORT 		|=  (1<<SDA);    			// internal Pull-Up on 
} 


#define I2C_SCL_LOW		I2C_PORT &= ~(1 << SCL)	// Clock Low Output
#define I2C_SCL_HIGH	I2C_PORT |=  (1 << SCL)	// Clock High Output

#define I2C_SDA_LOW		sda_low()				// Data signal  Low
#define I2C_SDA_HIGH	sda_high()				// Data signal High

#define	ACK				0						// Ack received
#define	NACK			1						// Ack not received


#define S_DELAY			_delay_loop_1(10);		// ca. 2.76s H-Time
#define M_DELAY			_delay_us(10);
#define DELAY			_delay_ms(10);			

/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
unsigned char i2c_try_scl(unsigned char start_trials)
{
	// wait 50 times if SCL is hold to low 
	while((!SCL) && (start_trials <50))
	{
		start_trials++;
		S_DELAY;
	}
	if(!SCL) return 0 ; return 1;
}

void i2c_master_stop(void);
/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

////////////////////////////////////////////////////////////////////////////////////////////
// I2C-Master Interface Standard Routinen
////////////////////////////////////////////////////////////////////////////////////////////
void i2c_master_init(void)
{
	I2C_DDR |=  (1<<SCL);  						// set SCL pin as output
		S_DELAY;
	I2C_SCL_HIGH;								// set SCL to H level
	I2C_SDA_HIGH;								// set SDA to H level
		S_DELAY;								// wait
	i2c_master_stop();
}
/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
void i2c_master_clk_toggle(void)
{
	i2c_try_scl(0x0A);

	I2C_SCL_HIGH;
		S_DELAY;
		S_DELAY;
	I2C_SCL_LOW;
 		S_DELAY;
}
/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
void i2c_master_start(void)
{
	// switch SDA from high to low; SCL is set to high
	i2c_try_scl(0x0A);

	I2C_SDA_HIGH;			// set SDA to high 
	//	S_DELAY;
	I2C_SCL_HIGH;			// set SCL to high	
		S_DELAY;
	I2C_SDA_LOW;			// set SDA to low
		M_DELAY;
		M_DELAY;
		M_DELAY;
	I2C_SCL_LOW;			// SCL auf low
		S_DELAY;
}
/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
void i2c_master_stop(void)		
{
	// switch SDA from low to high; SCL is set to high
	i2c_try_scl(0x0A);
		S_DELAY;
	I2C_SDA_LOW;			// set SDA to low
		S_DELAY;			// wait
	I2C_SCL_HIGH;			// set SCL to high
		S_DELAY;		
	I2C_SDA_HIGH;			// set SDA from low to high
		S_DELAY;
}
/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
unsigned char i2c_master_write(unsigned char b)
{	
	char i=0;
	for(i=0;i<8;i++)
	{
		if (b & 0x80)	
		{
			I2C_SDA_HIGH;
		}
		else
		{
			I2C_SDA_LOW;
		}
		b=b<<1;
		S_DELAY;
		i2c_master_clk_toggle();
	}
	// check ACK from slave
	I2C_SDA_HIGH;
	i2c_master_clk_toggle();		                                                                                                                   
	if(SDA==1)	
	{
		return 0;
	}
	else
	{ 
		return 1;
	}	
}
/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
unsigned char i2c_master_read(unsigned char a)
{	
	unsigned char i,c=0b00000000;
	I2C_SDA_HIGH;      

	for(i=0;i<8;i++)		
	{
		c=c<<1;				

		// Bitweise OR verknpfen mit dem Eingang, MSB zuerst
		if(SDA_LESE_PIN & (1 << SDA_LESE_BIT)) c|=1;			
		i2c_master_clk_toggle();
	}

	// ACK or NACK | ACK=0; NACK=1
	if(a==ACK)	{I2C_SDA_LOW;} else {I2C_SDA_HIGH;}  
	i2c_master_clk_toggle();
	return c;
}
